//
// NTSC composite encoder module
//
// Copyright(c) 2002 Tatsuyuki Satoh , All rights reserved
//
// Version : beta2
//
// Important !
//
// 1. License
//
// This program is freeware for non-commercial use. 
// The user of this source must assume all legal 
// responsibility in the use of this program. 
// This is not GPL.
//
// 2. Guarantee 
//
// An author does no guarantee about this program.
// You can use this under your own risk. 
// 
// 3. A rule to distribute.
//   Do either 1) or 2).
//   1)Include a perfect original source.
//   2)Include all these headers , and add document of 
//     modification points
//
// Histry:
// 2002.11. 5 : T.Satoh : inter-race & V timming fix,
//                        change define to parameter
//                        and many fix.
// 2002. 9. 3 : T.Satoh : Created
//
module ntsc_enc(
	T4SC_I,RESET,
	Y,V,U,HS_I,VS_I,
	VPOS_O,HPOS_O,
	FIELD,HSYNC_O,VSYNC_O,CSYNC_O,
	Y_O,C_O,V_O,
	SC_O,CB_O,WIN_O
);
//
// configration parameter
//
parameter interace_mode = 0;   // (NTSC=1) inter-race / non-inter-race
parameter intr_sync     = 0;   // (NTSC=0) internal HSYNC / external HSYNC
parameter VC_MAX        = 262; // (NTSC=263) vertical counter  
parameter HC_MAX        = 910; // (NTSC=910) horizonal counter 

input T4SC_I,RESET;
input [7:0] Y,V,U;
input HS_I,VS_I;
output FIELD,HSYNC_O,VSYNC_O,CSYNC_O;
output [7:0] Y_O,C_O,V_O;

output [9:0] HPOS_O;
output [8:0] VPOS_O;

output [1:0] SC_O;
output CB_O , WIN_O;

//
// OUTPUT LEVEL STEP (uV)
//
//parameter OUT_STEP = 6322; // 1.612Vpp@255step
parameter OUT_STEP = 4706;   // 1.200Vpp@255step

//
// Video timming gnnerator
//
// hpos : H display position (0-909)
// vpos : V display position (0-262)
// vout : 8bit composit V signal
// cout : 8bit S1 C signal
// yout : 8bit S2 Y signal
//

// Y fixed level
parameter Y_SYNC  = 0;
parameter Y_BLANK = 286000/OUT_STEP;      // 286mV
parameter Y_SETUP = (0/OUT_STEP)+Y_BLANK; // +0mV , (USA=54mV)
// C fixed level
parameter C_ZERO   = 0;
parameter C_CB30   = ( 52000/OUT_STEP); //  52mV 
parameter C_CB70   = (100000/OUT_STEP); // 100mV
parameter C_CBURST = (144000/OUT_STEP); // 144mVpp

//
// V-LINE state counters
//
parameter VC_PRE_EQUALIZING  =   3;
parameter VC_VERTICAL_SYNC   =   3+VC_PRE_EQUALIZING;
parameter VC_POST_EQUALIZING =   3+VC_VERTICAL_SYNC;
parameter VC_PRE_BLANK       =  10+VC_POST_EQUALIZING;
//
//
// H-LINE state counters
//
//-----------------------------------------------------------------------------
// H-LINE of display area
//
//  67clk : HorizontalSync     : 0V
//   9clk : Breezeway          : 256mV
//   2clk :  33% of ColorBurst : 256mV +- 52/0mV
//   2clk :  70% of ColorBurst : 256mV +-100/0mV
//  32clk : 100% of ColorBurst : 256mV +-144/0mV
//   2clk :  70% of ColorBurst : 256mV +-100/0mV
//   2clk :  33% of ColorBurst : 256mV +- 52/0mV
//  23clk : ColorBackPorch     : 256mV
// 752clk : pixel data         : 256mV +- Y +-C  : RED == 256mV+(56mV,321mV,608mV,264mV)
//  19clk : FrontPorch         : 256mV
//-----------------------------------------------------------------------------
//
parameter HC_HALF         = (HC_MAX/2);
//
parameter HC_HOR_SYNC     =  67;
parameter HC_BREEZE_WAY   =  10+HC_HOR_SYNC;
parameter HC_C_BURST1     =   2+HC_BREEZE_WAY;
parameter HC_C_BURST2     =   2+HC_C_BURST1; //  33%
parameter HC_C_BURST3     =  32+HC_C_BURST2; //  70%
parameter HC_C_BURST4     =   2+HC_C_BURST3; // 100%
parameter HC_C_BURST5     =   2+HC_C_BURST4; //  70%
parameter HC_C_BACK_PORCH =  22+HC_C_BURST5; //  33%
// WINDOW
parameter HC_FRONT_PORCH  = (HC_MAX-19);

parameter HC_HOR_SYNC1 = 0; // normal HSYNC on point
parameter HC_HOR_SYNC2 = HC_HOR_SYNC1-455; // 2x     HSYNC point
parameter HC_HOR_47US  = 67;  // HSYNC pulse 4.7us
parameter HC_HOR_23US  = 33;  // HSYNC pulse 2.3us

//
// internal regsisters
//
reg [7:0] ly,lv,lu;
reg [9:0] hc; // H-counter
reg [8:0] vc; // V-counter
reg fld;      // field
reg [1:0] sc; // subcarrier angle 0/90/180/270
reg [7:0] cout,yout,vout; // 8bit Y / C / composite video out

//
// V timming state
//
wire vsync_d    = (vc>=VC_PRE_EQUALIZING) & (vc<VC_VERTICAL_SYNC);
wire equalizing = (vc<VC_POST_EQUALIZING); // 2 x HSYNC lines
wire vblank_d   = vc<VC_PRE_BLANK;
wire vreturn_d  = vc==(VC_MAX-1);
// interace
wire half_hr    = interace_mode && ~fld & vreturn_d;                   // ODD  last scan line        = 454 end
wire half_pre   = fld & (vc==(VC_POST_EQUALIZING-1)); // EVEN after EQUALIZING line = 455 start

//
// H Timming state
//
wire hsync_d    = (hc<HC_HOR_SYNC);
wire cburst_d   = ~equalizing & (hc>=HC_BREEZE_WAY) & (hc<HC_C_BURST5);
wire cburst30_d = (hc<HC_C_BURST1) | (hc>=HC_C_BURST4);
wire cburst70_d = (hc<HC_C_BURST2) | (hc>=HC_C_BURST3);
wire hblank_d   = (hc<HC_C_BACK_PORCH)|(hc>=HC_FRONT_PORCH);
wire hreturn_d  = (hc==(HC_MAX-1)) | (half_hr&(hc==(HC_HALF-1)));
wire esync_p_d  = hc<HC_HALF ? hc<HC_HOR_23US            : hc<(HC_HALF+HC_HOR_23US);
wire vsync_p_d  = hc<HC_HALF ? hc>=(HC_HALF-HC_HOR_47US) : hc>=(HC_MAX-HC_HOR_47US);

//
// compsite states
//
wire blank_d  = vblank_d || hblank_d;
wire ncsync_d = vsync_d   ?  vsync_p_d  // VSYNC pulse
             : equalizing ? ~esync_p_d  // equalizing pulse
             : ~hsync_d;                // HSYNC pulse

//
// Y signal & SYNC mixer
//
wire [7:0] y_d;
assign y_d = ~ncsync_d ? Y_SYNC       // sync
	       : blank_d ? Y_BLANK      // blank
	       :           Y_SETUP + ly; // window

//
// COLOR signal encoder
//
wire [7:0] v_d,u_d,c_d;

assign v_d = ~cburst_d  ? lv // ColorSignal
           : C_ZERO;         // ColorBurst 0/-180

assign u_d = ~cburst_d  ? -lu // ColorSignal
           : cburst30_d ? C_CB30 // ColorBurst 90/-270
           : cburst70_d ? C_CB70
           :              C_CBURST;

assign c_d = blank_d & ~cburst_d ? C_ZERO  // blank
	        : (sc==2'b00) ?  v_d //   0'
	        : (sc==2'b01) ? -u_d //  90'
	        : (sc==2'b10) ? -v_d // 180'
	        :                u_d;// 270'

//
// Timming Generator & synchronizer
//
reg vsync , hsync , csync;
reg cburst , window;

reg  hs_s1 , hs_s2;
wire hs_iret = ~hs_s2 & hs_s1;
reg  vs_s1 , vs_s2;
wire vs_iret = ~vs_s2 & vs_s1;

reg vrs;
reg [8:0] vshift;
always @(posedge HS_I)
begin
	if(VS_I)
		vshift <= 240;
	else
		vshift <= vshift-1;
	vrs <= vshift==0;
end



always @(posedge T4SC_I or posedge RESET)
begin
	if( RESET )
	begin
		// Reset
		fld <= 0;
		vc  <= 0;
		hc  <= 0;
		sc  <= 0;
		vsync <= 0;
		hsync <= 0;
		csync <= 0;
		cburst <= 0;
		window <= 0;
		yout  <= 0;
		cout  <= 8'h80;
		vout  <= 0;
		ly <= 0;
		lv <= 0;
		lu <= 0;
	end else begin
	 	hs_s2 <= hs_s1;
	 	hs_s1 <= HS_I;
	 	if(sc == 2'b0)
		begin
			vs_s2 <= vs_s1;
//		 	vs_s1 <= VS_I;
		 	vs_s1 <= vrs;
		end

//		if(hreturn_d | hs_iret)
		if(hreturn_d)
		begin
			hc <= half_pre ? HC_HALF : 0;

//			if(vs_iret)
//				vc  <= VC_MAX-1;
//			else
//			if(vreturn_d | vs_iret)
			if(vreturn_d)
			begin
				vc  <= 0;
				fld <= interace_mode && ~fld;
			end else
				vc <= vc + 1;
	end else begin
			hc <= hc+1;
		end

		//External SYNC
		if(vs_iret && ~fld)
		begin
			hc <= 0;
			vc <= VC_MAX-1-4;
		end
		
		// subcarrier counter
		sc <= sc + 1;

		// Latch YUV input
		if( sc[0] == 1'b1)
		begin
			ly <= Y;
			lv <= V;
			lu <= U;
		end

		// SYNCs
		vsync <= vsync_d;
		hsync <= hsync_d;
		csync <= ncsync_d;

		// debug
		cburst <= cburst_d;
		window <= ~blank_d;

		// Y / C / V OUT
		yout  <= y_d;
		cout  <= 8'h80 + c_d;
		vout  <= y_d   + c_d;
	end
end

// output assign
assign HSYNC_O = hsync;
assign VSYNC_O = vsync;
assign CSYNC_O = csync;
assign Y_O = yout;
assign C_O = cout;
assign V_O = vout;
assign FIELD = fld;
assign VPOS_O = vc - VC_PRE_BLANK;
assign HPOS_O = hc - HC_C_BACK_PORCH;
assign SC_O = sc;
assign CB_O = cburst;
assign WIN_O = window;

endmodule

