/////////////////////////////////////////////////////////////////////////////
// FPGA PACMAN memory interface for Xilinx SpartanII
//
// Version : beta2
//
// Copyright(c) 2002,2003 Tatsuyuki Satoh , All rights reserved
//
// Important !
//
// This program is freeware for non-commercial use. 
// An author does no guarantee about this program.
// You can use this under your own risk. 
//
// 2003. 2. 6 beta2 Created
//
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// internal memory
/////////////////////////////////////////////////////////////////////////////

//
// Internal RAMs used 'BlockSelectRAM'
//
// 1.tilemap code RAM
// 2.tilemap color RAM
// 3.work & object RAM
// 4.object position RAM
// 5.object line RAM
// 6.sound waveform ROM
// 7.color lockup table ROM
// 8.palette ROM
//
// External ROM
//
// 9.CPU code ROM
// 10.object pattern ROM
//

module xmemory(
//
  CLK_12M,
// Sahred Address & Data(Out) Bus
  AB,DB_I,DB_O,
  TILE_CS,TILE_WE, COL_CS,COL_WE, WORK_CS,WORK_WE,
  PWE,PDI,PDO,
  LA,LDO,LDI,LCS,LWE,
//  SA,S0DI,S0DO,S1DI,S1DO,SWE,
  WA,WDO,
  CLUTA,CLUT_DO,
  PAL_A,PAL_DO
);

/////////////////////////////////////////////////////////////////////////////
// module I/O assign
/////////////////////////////////////////////////////////////////////////////
input CLK_12M;
input [11:0] AB;
input [7:0] DB_I;
output [7:0] DB_O;

// TileCode RAM
input TILE_CS,TILE_WE;
// TileColor RAM
input COL_CS,COL_WE;
// Work/ObjectAttribute RAM
input WORK_CS,WORK_WE;
// Object Position RAM
input PWE;
input [7:0] PDI;
output [7:0] PDO;
// Object Line RAM
input [7:0] LA;
input [3:0] LDI;
output [3:0] LDO;
input LCS,LWE;
// sound Register RAM
// input [3:0] SA;
// output [3:0] S0DO , S1DO;
// input  [3:0] S0DI , S1DI;
// input SWE;
// sound WaveForm ROM
input [8:0] WA;
output [3:0] WDO;
//ColorLookup ROM
input [7:0] CLUTA;
output [3:0] CLUT_DO;
input [3:0] PAL_A;
output [7:0] PAL_DO;

/////////////////////////////////////////////////////////////////////////////
// Video RAM
/////////////////////////////////////////////////////////////////////////////

wire [7:0] tile_do , col_do , wram_do;
assign DB_O = tile_do | col_do | wram_do;

//
// 1.tilemap code RAM
//
// Area    : 4000-43ff
// Size    : 8bit x 1024word
// Address : AB[9:0]
// Dout    : DB_O (shared)
// Din     : DB_I
// CS      : TILE_CS
// WE      : TILE_WE
// 
RAMB4_S4 RAM_4N (.CLK(CLK_12M),.ADDR(AB[9:0]),.DO(tile_do[7:4]),.DI(DB_I[7:4]),.EN(1'b1),.WE(TILE_WE),.RST(~TILE_CS));
RAMB4_S4 RAM_4K (.CLK(CLK_12M),.ADDR(AB[9:0]),.DO(tile_do[3:0]),.DI(DB_I[3:0]),.EN(1'b1),.WE(TILE_WE),.RST(~TILE_CS));

//
// 2.tilemap color RAM
//
// parts   : 
// Area    : 4400-47ff
// Size    : 8bit x 1024word
// Address : AB[9:0]
// Dout    : DB_O (shared)
// Din     : DB_I
// CS      : COL_CS
// WE      : COL_WE
// 
RAMB4_S4 RAM_4P (.CLK(CLK_12M),.ADDR(AB[9:0]),.DO(col_do[7:4]),.DI(DB_I[7:4]),.EN(1'b1),.WE(COL_WE),.RST(~COL_CS));
RAMB4_S4 RAM_4L (.CLK(CLK_12M),.ADDR(AB[9:0]),.DO(col_do[3:0]),.DI(DB_I[3:0]),.EN(1'b1),.WE(COL_WE),.RST(~COL_CS));

//
// 3.work & object RAM
//
// parts   : 4R,4M
// Area    : 4c00-4fff
// Size    : 8bit x 1024word
// Address : AB[9:0]
// Dout    : DB_O (shared)
// Din     : DB_I
// CS      : WORK_CS
// WE      : WORK_WE
// 
RAMB4_S4 RAM_4R (.CLK(CLK_12M),.ADDR(AB[9:0]),.DO(wram_do[7:4]),.DI(DB_I[7:4]),.EN(1'b1),.WE(WORK_WE),.RST(~WORK_CS));
RAMB4_S4 RAM_4M (.CLK(CLK_12M),.ADDR(AB[9:0]),.DO(wram_do[3:0]),.DI(DB_I[3:0]),.EN(1'b1),.WE(WORK_WE),.RST(~WORK_CS));

//
// 4.object position RAM
//
// parts   : 3F,3E
// Area    : 5060-506f
// Size    : 8bit x 16word
// Address : AB[3:0]
// Dout    : PDO (separate)
// Din     : PDI (separate)
// CS      : always select
// WE      : PWE
// 

//
// 5.object line RAM
//
// parts   : (1A,1B,1C,1D) or (2A,2B,2B,2D)
// Area    : none
// Size    : 4bit x 256word
// Address : LA
// Dout    : LDO (separate)
// Din     : LDI (separate)
// CS      : LCS
// WE      : LWE (6MHz)
//
// note : LDO is latched LWE fall,so read data should be setup before LWE fall.
//
wire [8:0] pram_a9  =  { 5'b10000,AB[3:0] };
wire [9:0] lram_a10 = { 2'b00, LA };
RAMB4_S4_S8 pram_lram (
  .CLKA(~CLK_12M) ,.ADDRA(lram_a10),.DOA(LDO),.DIA(LDI),.WEA(LWE),.ENA(LCS),.RSTA(~LCS),// LRAM
  .CLKB(CLK_12M),.ADDRB(pram_a9),.DOB(PDO),.DIB(PDI),.ENB(1'b1),.WEB(PWE),.RSTB(1'b0)  // PRAM
);
//
// 6.sound waveform ROM
//
// parts   : 1M
// Area    : none
// Size    : 4bit x 256word
// Address : WA
// Dout    : WDO
// CS      : always select
//
wire [10:0] wfa = {1'b0,WA,1'b0}; // resize to 256x8bit
RAMB4_S4 WAVEFORM_ROM00 (.CLK(CLK_12M),.ADDR(wfa),.DO(WDO),.DI(4'h0),.EN(1'b1),.WE(1'b0),.RST(1'b0));


//
// 7.color lockup table ROM
// 8.palette ROM
//

wire [8:0] clut_a9 = { 1'b0,CLUTA }; // CLUT
wire [7:0] clut_do8;
assign CLUT_DO = clut_do8[3:0];
wire [8:0] pal_a9 = { 5'b10000,PAL_A }; //palette
RAMB4_S8_S8 clut_rom00 (
	.CLKA( CLK_12M),.ADDRA(clut_a9),.DOA(clut_do8),.DIA(8'h0),.WEA(1'b0),.ENA(1'b1),.RSTA(1'b0),
	.CLKB(~CLK_12M),.ADDRB(pal_a9),.DOB(PAL_DO),.DIB(8'h0),.WEB(1'b0),.ENB(1'b1),.RSTB(1'b0));
//
// ----- PALETTE RAM async. version -----
//
// BSRAM version make half pixel delay to RGB output.
// fill a ROM code here if you dislike that.
//
//function [7:0] plut_rom;
//input [3:0] clut;
//begin
//	case ( clut )
//	 0:plut_rom = 8'hxx;
//	 1:plut_rom = 8'hxx;
//	 2:plut_rom = 8'hxx;
//	 3:plut_rom = 8'hxx;
//	 4:plut_rom = 8'hxx;
//	 5:plut_rom = 8'hxx;
//	 6:plut_rom = 8'hxx;
//	 7:plut_rom = 8'hxx;
//	 8:plut_rom = 8'hxx;
//	 9:plut_rom = 8'hxx;
//	10:plut_rom = 8'hxx;
//	11:plut_rom = 8'hxx;
//	12:plut_rom = 8'hxx;
//	13:plut_rom = 8'hxx;
//	14:plut_rom = 8'hxx;
//	15:plut_rom = 8'hxx;
//	endcase
//end
//endfunction
//assign PAL_DO = plut_rom(PAL_A);
//
//RAMB4_S8 clut_rom00 (
//	.CLK(~pclk),.ADDR(clut_a),.DO(clut_d),.DI(8'h0),.WE(1'b0),.EN(1'b1),.RST(1'b0));
//
// ----- PALETTE RAM async. version -----


endmodule
